
--[[--------------------------------------------------------------------------

    This file is part of lunit 0.4.

    For Details about lunit look at: http://www.mroth.net/lunit/

    Author: Michael Roth <mroth@nessie.de>

    Copyright (c) 2006-2008 Michael Roth <mroth@nessie.de>

    Permission is hereby granted, free of charge, to any person 
    obtaining a copy of this software and associated documentation
    files (the "Software"), to deal in the Software without restriction,
    including without limitation the rights to use, copy, modify, merge,
    publish, distribute, sublicense, and/or sell copies of the Software,
    and to permit persons to whom the Software is furnished to do so,
    subject to the following conditions:

    The above copyright notice and this permission notice shall be 
    included in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--]]--------------------------------------------------------------------------



--[[

      begin()
        run(testcasename, testname)
          err(fullname, message, traceback)
          fail(fullname, where, message, usermessage)
          pass(testcasename, testname)
      done()

      Fullname:
        testcase.testname
        testcase.testname:setupname
        testcase.testname:teardownname

--]]


--[[
  Changed:
    "lunit-console" was renamed to "lunit-nscripter".
    Calling "printformat" was replaced to "writeout(string.format(format,...))" because of NScripter's unclearly behaviours.
    Calling "io.write" was replaced to "writeout()" because NScripter is not good at printing ASCII characters.
    "total_tests" scope was moved out of "function" to show the percentage of passing tests by test.
    "NSExec" was inserted for printing the results by NScripter.
--]]

require "lunit"

module( "lunit-nscripter", package.seeall )


local function printformat(format, ...)
  io.write( string.format(format, ...) )
end

local linecount = 0
local maxchars = 46*2 -- bytes
local maxlines = 16+10

local function tomb(str)
  -- Replace ASCII character to multi-byte character.
  local mb = false
  return string.gsub(str, ".", function (c)
    bc = string.byte(c)
    if mb then
      -- multi-byte character following
        mb = false
        return c
    else
      -- multi-byte character leading
      if bc > 127 then
        mb = true
        return c
      end
      -- ascii character
      if string.byte("0") <= bc and bc <= string.byte("9") then
        return string.char(string.byte("O"), string.byte("O", 2) + bc - string.byte("0"))
      elseif string.byte("a") <= bc and bc <= string.byte("z") then
        return string.char(string.byte(""), string.byte("", 2) + bc - string.byte("a"))
      elseif string.byte("A") <= bc and bc <= string.byte("Z") then
        return string.char(string.byte("`"), string.byte("`", 2) + bc - string.byte("A"))
      elseif c == "-" then
        return "]"
      elseif c == "+" then
        return "{"
      elseif c == "/" then
        return "^"
      elseif c == "*" then
        return ""
      elseif c == "=" then
        return ""
      elseif c == "_" then
        return "Q"
      elseif c == " " then
        return "@"
      elseif c == "@" then
        return ""
      elseif c == "\\" then
        return ""
      elseif c == "(" then
        return "i"
      elseif c == ")" then
        return "j"
      elseif c == "[" then
        return "m"
      elseif c == "]" then
        return "n"
      elseif c == "{" then
        return "o"
      elseif c == "}" then
        return "p"
      elseif c == "<" then
        return ""
      elseif c == ">" then
        return ""
      elseif c == "\t" then
        return "@@"
      elseif c == "." then
        return "D"
      elseif c == "," then
        return "A"
      elseif c == ":" then
        return "F"
      elseif c == ";" then
        return "G"
      elseif c == "'" then
        return "f"
      elseif c == "`" then
        return "e"
      elseif c == "\"" then
        return "h"
      elseif c == "%" then
        return ""
      elseif c == "$" then
        return ""
      elseif c == "#" then
        return ""
      elseif c == "?" then
        return "H"
      elseif c == "!" then
        return "I"
      elseif c == "\n" then
        return "\n"
      else
        return ""
      end
    end
  end)
end

local function writeout(str)
  -- Put texts by line.
  linefeeded = false
  for line in string.gmatch(tomb(str), "([^\n]*\n?)") do
    if line == "\n" then
      if linefeeded then
        NSExec('_br')
        linefeeded = false
      else
        NSExec('_puttext ""')
        linefeeded = true
      end
      linecount = linecount + 1
    else
      local lineend = "/"
      if string.sub(line, -1) == "\n" then
        line = string.sub(line, 1, string.len(line)-1)
        lineend = ""
        linefeeded = true
        linecount = linecount + 1
      else
        linefeeded = false
      end
      NSExec('_puttext "' .. line .. lineend .. '"')
      local increased = math.floor(string.len(line) / maxchars)
      linecount = linecount + increased
    end
    -- Need the next page?
    if linecount > maxlines - 3 then
      NSExec('_puttext "\\/"')
      linecount = 0
    end
  end
end

local columns_printed = 0

local function writestatus(char)
  if columns_printed == 0 then
    writeout("    ")
  end
  if columns_printed == 60 then
    writeout("\n    ")
    columns_printed = 0
  end
  writeout(char)
  columns_printed = columns_printed + 1
end


local msgs = {}
local total_tests = 0

function begin()
  NSExec('_setwindow 8,16,46,16+10,13,16,0,1,20,1,1,#999999,0,0,639,479')
  local total_tc = 0
  total_tests = 0

  for tcname in lunit.testcases() do
    total_tc = total_tc + 1
    for testname, test in lunit.tests(tcname) do
      total_tests = total_tests + 1
    end
  end

  writeout(string.format("Loaded testsuite with %d tests in %d testcases.\n\n", total_tests, total_tc))
end


function run(testcasename, testname)
  -- NOP
end


function err(fullname, message, traceback)
  writestatus("E")
  msgs[#msgs+1] = "Error! ("..fullname.."):\n"..message.."\n\t"..table.concat(traceback, "\n\t") .. "\n"
end


function fail(fullname, where, message, usermessage)
  writestatus("F")
  local text =  "Failure ("..fullname.."):\n"..
                where..": "..message.."\n"

  if usermessage then
    text = text .. where..": "..usermessage.."\n"
  end

  msgs[#msgs+1] = text
end


function pass(testcasename, testname)
  writestatus(".")
end



function done()
  writeout(string.format("\n\n%d Assertions checked.\n", lunit.stats.assertions ))

  for i, msg in ipairs(msgs) do
    writeout(string.format( "%3d) %s\n", i, msg ))
  end

  writeout(string.format("Testsuite finished (%d passed, %d failed, %d errors).\n",
      lunit.stats.passed, lunit.stats.failed, lunit.stats.errors ))
  NSExec('_erasetextwindow 0')
  NSExec('_lsp 1,":c;>640,20,#ff0000",0,450')
  NSExec('_lsp 0,":c;>' .. (640*lunit.stats.passed/total_tests) .. ',20,#00ff00",0,450')
  NSExec('_print 1')
end





